local thread = love.thread.newThread("iothread.lua")
local ichannel = love.thread.getChannel("i")
local ochannel = love.thread.getChannel("o")

local asyncfs = {}
local callbackid = 0

asyncfs.callbacks = {}

function love.threaderror(thread, message)
    error("thread error\n\n"..message)
end

function asyncfs.update()
    for i = 1, ochannel:getCount() do
        local message = ochannel:pop()
        if message and asyncfs.callbacks[message.id] then
            -- print(message.id, message.type)
            asyncfs.callbacks[message.id](message)
            asyncfs.callbacks[message.id] = nil
        end
    end
end

--- @async
function asyncfs.send(message, callback)
    -- print("sending", message.type)
    callbackid = callbackid + 1
    message.id = callbackid
    asyncfs.callbacks[callbackid] = callback
    ichannel:push(message)
end

--- @async
function asyncfs.read(filename, callback)
    asyncfs.send({
        type = "read",
        filename = filename,
    }, callback)
end

--- @async
function asyncfs.write(filename, data, callback)
    asyncfs.send({
        type = "write",
        filename = filename,
        data = data,
    }, callback)
end

thread:start()

return asyncfs