return function(world, region, options)
    options = options or {}
    local x = math.floor(region.x or 0)
    local y = math.floor(region.y or 0)
    local width = region.width or 20
    local height = region.height or 20

    local squish = options.squish or 10
    local hillamp = options.hillamp or 5
    local mountainamp = options.mountainamp or 5
    local oceanamp = options.oceanamp or 5
    local sealevel = options.sealevel or 6
    local treefrequency = options.treefrequency or 0.2
    local rockfrequency = options.rockfrequency or 0.1

    local grass = options.grass or 0
    local soil = options.soil or 1
    local rock = options.rock or 2
    local sand = options.sand or 5
    local roots = options.roots or 6

    local sandheight = options.sandheight or 5
    local soilheight = options.soilheight or 5
    -- features
    for i = 1, width do
        -- get final x
        local fx = i + x - 1
        -- get column data
        local col = world:x(fx)
        -- set tile heights
        local xpos = (x + i) * squish
        local mountainheight = (love.math.noise(xpos / 1000, world.seed) - 0.5) * 2
        local oceanheight = (love.math.noise(xpos / 10000, world.seed) - 0.5) * 2
        col.groundheight = math.floor((love.math.noise(xpos / 100, world.seed) - 0.5) * 2 * hillamp * (1-math.abs(mountainheight)) + mountainheight * mountainamp * 3 + oceanheight * oceanamp * 3)
        -- place tree
        col.tree = love.math.noise(xpos, world.seed)
        col.rock = love.math.noise(world.seed, xpos*5)
    end
    for ix = 1, width do
        -- get final x
        local fx = ix + x - 1
        -- get column data
        local col = world:x(fx)
        local groundheight = col.groundheight
        local tilesunder = 0
        for iy = 1, height do
            local tile
            local fy = iy + y - 1
            local rand = love.math.noise(fx / 5, world.seed)
            if fy >= groundheight then
                tilesunder = fy - groundheight
                if groundheight > sealevel - rand*2 - sandheight and tilesunder < rand*2 + sandheight - math.abs(groundheight - sealevel) then
                    tile = sand
                else
                    if tilesunder == 0 and fy <= sealevel then
                        tile = grass
                        if col.tree <= treefrequency then
                            local nextcol = world:x(fx+1)
                            local prevcol = world:x(fx-1)
                            if col.tree <= treefrequency/2 and nextcol.groundheight == groundheight then
                                world:spawn(fx * 8 + 8, groundheight * 8, "bigtree")
                                nextcol.tree = 9999
                                tile = roots
                                world:set(fx + 1, nextcol.groundheight, roots)
                            -- elseif col.tree <= treefrequency/2 and prevcol.groundheight == groundheight then
                            --     world:spawn(fx * 8, groundheight * 8, "bigtree")
                            --     tile = roots
                            --     world:set(fx-1, prevcol.groundheight, roots)
                            else
                                world:spawn(fx * 8 + 4, groundheight * 8, "tree")
                                tile = roots
                            end
                        end
                        if col.tree == 9999 then
                            tile = roots
                        end
                    else
                        if tilesunder < soilheight + rand*2 then
                            tile = soil
                        else
                            tile = rock
                        end
                    end
                end
            end
            if fy == groundheight and fy <= sealevel then
                if col.rock <= rockfrequency and col.tree > treefrequency then
                    world:spawn(fx * 8 + 4, groundheight * 8, "rock")
                end
            end
            if not tile then
                if fy == sealevel then
                    tile = 3
                elseif fy > sealevel then
                    tile = 4
                end
            end
            world:set(fx, fy, tile)
        end
    end
end