local loader = {}

loader.cache = {
    [_G] = {}
}

function loader.image(src)
    if not loader.cache[src] then
        loader.cache[src] = love.graphics.newImage(src)
    end
    return loader.cache[src]
end

function loader.shader(src)
    if not loader.cache[src] then
        loader.cache[src] = love.graphics.newShader(src)
    end
    return loader.cache[src]
end

--- @param modname string
--- @param env table?
--- @param errorhandler function?
function loader.import(modname, env, errorhandler)
    env = env or _G
    errorhandler = errorhandler or error
    local modpath = modname:gsub("%.+", "/")
    if loader.cache[env] and loader.cache[env][modpath] then
        return loader.cache[env][modpath]
    end
    if not loader.cache[env] then
        loader.cache[env] = {}
    end
    local cache = loader.cache[env]
    local searchedpaths = {}
    local requirepaths = love.filesystem.getRequirePath()
    for a in requirepaths:gmatch("[^;]+") do
        local path = a:gsub("%?", modpath)
        local chunk, errmsg = loadfile(path, "bt", env)
        if chunk then
            local ret = {xpcall(chunk, function(msg)
                errorhandler(msg)
            end)}
            local success = table.remove(ret, 1)
            cache[modpath] = ret
            return unpack(ret)
        elseif errmsg and not errmsg:find("^cannot open "..path) then
            errorhandler(errmsg)
        end
        searchedpaths[#searchedpaths+1] = path
    end
    local pref = "        searched in "
    errorhandler("module '"..modname.."' not found:\n"..pref..table.concat(searchedpaths, "\n"..pref))
end

return loader