local obj_metatable = {}
local player_metatable = {}
local typecache = {
    player = require "types.player"
}
local valuetypes = {
    boolean = true,
    string = true,
    number = true,
    ["nil"] = true,
    table = true
}
local globalstatics = {
    type = true,
    uid = true
}

function obj_metatable:__index(key)
    local objecttype = typecache[rawget(self, "type")]
    local static = rawget(objecttype, key)
    if static ~= nil then
        return static
    end
    return rawget(self, key)
end

player_metatable.__index = typecache.player

function obj_metatable:__newindex(key, value)
    if rawget(globalstatics, key) or self[key] ~= nil then
        error("Cannot set static property of object (key '"..key.."')")
    end
    if rawget(valuetypes, type(key)) and rawget(valuetypes, type(value)) then
        return rawset(self, key, value)
    end
    error("Invalid object property type (key '"..type(key).."' value '"..type(value).."')")
end

player_metatable.__newindex = obj_metatable.__newindex

return function(obj, isplayer)
    if isplayer then
        return setmetatable(obj, player_metatable)
    end
    if not typecache[obj.type] then
        typecache[obj.type] = require("types."..obj.type)
    end
    return setmetatable(obj, obj_metatable)
end