local serializer = {}

function serializer.serializevalue(obj, references, backreferences)
    local refs = references
    local backrefs = backreferences
    if type(obj) == "number" then
        return "n"..love.data.pack("string", "s1", tostring(obj))
    end
    if type(obj) == "string" then
        return "s"..love.data.pack("string", "s4", obj)
    end
    if type(obj) == "boolean" then
        if obj then
            return "t"
        else
            return "f"
        end
    end
    if type(obj) == "table" then
        if backrefs[obj] then
            return backrefs[obj]
        else
            local ref = {"r"}
            local count = 0
            for key, value in pairs(obj) do
                local skey = serializer.serializevalue(key, refs, backrefs)
                local svalue = serializer.serializevalue(value, refs, backrefs)
                if skey and svalue then
                    ref[#ref+1] = skey
                    ref[#ref+1] = svalue
                    count = count + 1
                end
            end
            ref[1] = ref[1]..love.data.pack("string", "i4", count)
            refs[#refs+1] = table.concat(ref, "")
            local ret = "a"..love.data.pack("string", "i4", #refs)
            backrefs[obj] = ret
            return ret
        end
    end
    error("wuh oh")
end

function serializer.serialize(obj)
    local refs = {}
    local serialized = serializer.serializevalue(obj, refs, {})
    return table.concat(refs, "")..serialized
end

function serializer.deserializevalue(str, position, references)
    local pos = position
    local refs = references
    local vtype = str:sub(pos, pos)
    -- print(vtype)
    if vtype == "t" then
        return true, pos+1
    elseif vtype == "f" then
        return false, pos+1
    elseif vtype == "n" then
        local num, index = love.data.unpack("s1", str, pos+1)
        -- print(num)
        return tonumber(num), index
    elseif vtype == "s" then
        local str2, index = love.data.unpack("s4", str, pos+1)
        -- print(str2)
        return str2, index
    elseif vtype == "a" then
        local refindex, index = love.data.unpack("i4", str, pos+1)
        return refs[refindex], index
    end
end

function serializer.deserializeref(str, position, references)
    local pos = position
    local refs = references
    local vtype = str:sub(pos, pos)
    if vtype == "r" then
        -- print"NEWREF"
        local ret = {}
        local count, index = love.data.unpack("i4", str, pos+1)
        for i = 1, count do
            local key
            local value
            key, index = serializer.deserializevalue(str, index, refs)
            value, index = serializer.deserializevalue(str, index, refs)
---@diagnostic disable-next-line: need-check-nil
            ret[key] = value
        end
        refs[#refs+1] = ret
        return true, index
    else
        -- print "ENDREF"
        return false, pos
    end
end

function serializer.deserialize(str)
    local refs = {}
    local continue, index = true, 1
    while continue do
---@diagnostic disable-next-line: cast-local-type
        continue, index = serializer.deserializeref(str, index, refs)
    end
    return serializer.deserializevalue(str, index, refs)
end

function serializer.save(obj, path)
    return love.filesystem.write(path, serializer.serialize(obj))
end

function serializer.load(path)
    local content = love.filesystem.read("string", path)
    if content then
        return serializer.deserialize(content)
    end
    return nil
end

return serializer