local self = {}

function self:draw(world, offsetx, offsety)
    if self.image and self.x and self.y then
        local tx = math.ceil(self.x * world.scale) / world.scale
        local ty = math.ceil(self.y * world.scale) / world.scale
        local xpos = tx
        local ypos = ty
        offsetx = offsetx or 0
        offsety = offsety or 0
        local image = loader.image(self.image)
        local fx = xpos
        if self.mirrored then
            love.graphics.scale(-1, 1)
            fx = -xpos
        end
        if self.quad then
            local quad = love.graphics.newQuad(self.quad.x, self.quad.y, self.quad.width, self.quad.height, image:getWidth(), image:getHeight())
            love.graphics.draw(image, quad, fx - self.quad.width/2 + offsetx, ypos - self.quad.height + offsety)
        else
            love.graphics.draw(image, fx - image:getWidth()/2 + offsetx, ypos - image:getHeight() + offsety)
        end
        if self.mirrored then
            love.graphics.scale(-1, 1)
        end
    end
end

function self:init(world)
    self.xv = 0
    self.yv = 0

    self.canjump = false
    self.swimming = false
end

function self:move(world, up, down, left, right)
    local speed = 0.3
    if self.canjump then
        speed = 0.5
    end
    if left then
        if self.xv > -2 then
            self.xv = self.xv - speed
        end
    elseif right then
        if self.xv < 2 then
            self.xv = self.xv + speed
        end
    elseif self.xv > 0 then
        self.xv = self.xv - speed/3
    elseif self.xv < 0 then
        self.xv = self.xv + speed/3
    end

    if math.abs(self.xv) < speed / 2 then
        self.xv = 0
    end
    self.canjump = false

    self.x = self.x + self.xv
    
    if self.xv < 0 then
        self.mirrored = true
    elseif self.xv > 0 then
        self.mirrored = false
    end
    local tilex, tiley = world:getspace(self.x - self.width/2 + 0.1, self.y - 1)
    if world:get(tilex, tiley).collision == "solid" or world:get(world:getspace(self.x - self.width/2 + 0.1, self.y - self.height + 1)).collision == "solid" then
        if self.xv < 0 or self.yv < 0 then
            self.x = tilex * 8 + 8 + self.width / 2
            self.xv = 0
        else
            self.canjump = true
            self.xv = self.xv + 1
            self.yv = 1
        end
    end
    local tilex, tiley = world:getspace(self.x + self.width/2 - 0.1, self.y - 1)
    if world:get(tilex, tiley).collision == "solid" or world:get(world:getspace(self.x + self.width/2 - 0.1, self.y - self.height + 1)).collision == "solid" then
        if self.xv > 0 or self.yv < 0 then
            self.x = tilex * 8 - self.width / 2
            self.xv = 0
        else
            self.yv = 1
            self.xv = self.xv - 1
            self.canjump = true
        end
    end

    self.swimming = false

    local ctile = world:get(world:getspace(self.x, self.y - self.height / 2))
    if ctile.collision == "water" then
        self.swimming = true
    end

    if self.swimming then
        if up or (self.floats and not down) then
            if up then
                if self.yv > -3 then
                    self.yv = self.yv - 0.2
                elseif self.yv > 3 then
                    self.yv = self.yv + 0.1
                end
            else
                if self.yv > -1.5 then
                    self.yv = self.yv - 0.1
                elseif self.yv > 1.5 then
                    self.yv = self.yv + 0.05
                end
            end
        else
            if self.yv < 3 then
                self.yv = self.yv + 0.2
            elseif self.yv > 3 then
                self.yv = self.yv - 0.1
            end
        end
    else
        if self.yv < 6 then
            self.yv = self.yv + 0.5
        end
    end

    self.y = self.y + self.yv
    
    if self.yv < 0 then
        local tilex, tiley = world:getspace(self.x, self.y - self.height)
        if world:get(tilex, tiley).collision == "solid" then
            self.y = tiley * 8 + 8 + self.height
            self.yv = 0
        end
    elseif self.yv > 0 then
        local tilex, tiley = world:getspace(self.x, self.y)
        if world:get(tilex, tiley).collision == "solid" then
            self.y = tiley * 8
            self.yv = 0
            self.canjump = true
        end
    end

    local etile = world:get(world:getspace(self.x, self.y - self.height / 2))
    if etile.collision ~= "water" and self.swimming and self.yv <= 0 then
        if up then
            self.yv = -4
        else
            self.yv = self.yv + 1
        end
    end

    if not self.swimming then
        if up and self.canjump then
            self.yv = -6
        end
    end
end

return self