local base = require "types.base"

local self = {}

self.width = 6
self.height = 6
self.inventorysize = 15
self.hotbarsize = 5
self.image = "sprites/bart.png"

self.slotimg = loader.image("sprites/slot_alt.png")
self.selectedslotimg = loader.image("sprites/selected_slot.png")

function self:init(world)
    base.init(self, world)

    self.currentslot = 1
    self.inventory = {}

    self.attack = 1
    self.defense = 1
end

self.draw = base.draw

function self:update(world, dt)
    base.move(self, world, world.controls.up, world.controls.down, world.controls.left, world.controls.right)

    world.camerax = self.x - love.graphics.getWidth() / 2 / world.scale
    world.cameray = self.y - love.graphics.getHeight() / 2 / world.scale

    if world:isDown(2) then
        local mousetx, mousety = world:getspace(world.mousex, world.mousey)
        world:set(mousetx, mousety, 2)
    end

    if world:isDown(1) then
        world:set(world:getspace(world.mousex, world.mousey))
    end

    if world:isDown "0" then
        local newobj = world:spawn(self.x, self.y, "item")
        newobj.xv = love.math.random(-3.9, 3.9)
        newobj.yv = love.math.random(-3.9, 3.9)
    end
end

function self:keypressed(world, key)
    local slot = tonumber(key)
    if slot and slot >= 1 and slot <= self.hotbarsize then
        self.currentslot = slot
    end
end

function self:wheelmoved(world, x, y)
    if x - y < 0 then
        self.currentslot = self.currentslot - 1
    else
        self.currentslot = self.currentslot + 1
    end
    if self.currentslot > self.hotbarsize then
        self.currentslot = self.hotbarsize
    end
    if self.currentslot < 1 then
        self.currentslot = 1
    end
end

function self:drawtop(world)
    love.graphics.setLineWidth(1)
    love.graphics.setColor(1, 1, 1)
    love.graphics.rectangle("line", math.floor(world.mousex/8)*8 - 0.5, math.floor(world.mousey/8)*8 - 0.5, 9, 9)
end

function self:drawui(world)
    local w = math.floor(world:viewWidth() / 2 * world.scale) / world.scale
    local h = world:viewHeight() - 6
    local sp = 16
    for i = 1, self.hotbarsize do
        local offsety = 0
        love.graphics.draw(self.slotimg, w - (self.hotbarsize-1)*sp/2 + sp * (i-1), h + offsety, 0, 1, 1, 7, 13)
        if i == self.currentslot then
            love.graphics.draw(self.selectedslotimg, w - (self.hotbarsize-1)*sp/2 + sp * (i-1), h + offsety, 0, 1, 1, 7, 13)
        end
    end
end

return self